; ********************************************************************************
; Addressable RGB PWM LED driver with serial comms interface
; V2 Production Code 1.0.2 (14/06/2006)
; Pete Griffiths (c) 2006
; picprojects.org.uk
; For non-commercial use only
; Written and assembled using MPLAB v7.31
;
; ********************************************************************************
; Note: For 12F629 and 12F675
; All timings are based on the internal 4Mhz PIC oscillator.  The OSCCAL value is read
; from program memory at 0x3FF and written to the OSCAL register.  This value must be
; present and correct. 
; a) If it is not present the code will crash.
; b) If it's incorrect the bit rate for the serial comms may be too far out to receive
;    data reliably if at all.
;
; This does not apply to 12F683 as it doesn't use this method to calibrate the oscillator
;
; ********************************************************************************
; Ensure that GPIO3 is either driven or pulled up but NOT LEFT FLOATING
;	        ___ ___
;  	   Vdd -|1 ^ 8|- Vss
; 	   Rxd -|2   7|- red
;  unused. tie to  Vdd -|3   6|- green
;  unused. tie to  Vdd -|4   5|- blue
;	        
; ********************************************************************************
;
; Device addressing:
; ~~~~~~~~~~~~~~~~~
; Device's can be addressed individualy using their unique Address
; As a group using the Multicast address for the device's group
; All devices using the Broadcast address.
; 
; Address	Group MCast
; 0x00-0x0F	0x80	16 unique addresses in group
; 0x10-0x1F	0x90	16 unique addresses in group
; 0x20-0x2F	0xA0	16 unique addresses in group
; 0x30-0x3F	0xB0	16 unique addresses in group
; 0x40-0x4F	0xC0	16 unique addresses in group
; 0x50-0x5F	0xD0	16 unique addresses in group
; 0x60-0x6F	0xE0	16 unique addresses in group
; 0x70-0x7F	0xF0	16 unique addresses in group
; 0xFE	n/a	command packet
; 0xFF	n/a	All devices Broadcast address
; 
;
; ********************************************************************************
; Commands
; ~~~~~~~~~~~~~~~~~~~~~~~
; Address 0xFE is used to indicate the packet contains a command.
; All devices attached to the serial bus respond to a command packet.
; The TransferMemory, SelectMemory, WriteMode and Reset commands will affect all
; devices on the bus.  For example the selectMemory command will cause all devices
; on the bus to set their active RGB memory to the value specified by the command data.
; The address change command is processed by all devices but will only change the 
; address of the device specified in the command data.
;
; ********************************************************************************
; SelectMemory, TransferMemory and Write mode command detail
; ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
; In Write Direct mode, RGB data received for this device is written directly to the LEDs
; In Write Memory mode, RGB data received for this device is written to the active memory only.
; To transfer the contents of any memory to the LEDs, send a Transfer Memory command with
; the number of the memory to transfer.  This command is applied to all devices which means
; a) the LED update is synchronous across all devices
; b) the selected memory of all devices should contain the required data before issuing the command
;
; Write Direct mode
;  -Received RGB Data --> LEDs
;  -At reset the device is always set to WriteThrough mode
;
; Write memory mode
;  -Received RGB Data --> active RGB memory
;
; Transfer memory to LEDs
;  Command 0x01 ComData1 [0::cMAXMEM]
;  -transfer RGB memory [value in ComData1] to LEDs
;
; Select active memory for writing
;  Command 0x02 ComData1 [0::cMAXMEM]
;  -set active RGB Memory to value in ComData1
;
; After a reset (power-on or via the reset command)
; 1. all RGB memories are set to 0x00
; 2. RGB Memory 0 is active
; 3. Write Direct mode is enabled
;
;
; ********************************************************************************
; MPLAB listing config

  LIST b=16, n=0, t=ON, st=On, c=80
; b  - tab spacing
; n  - lines per page
; c  - column width
; t  - truncate lines on, wrap lines off
; st - print symbol table on/off

  IFDEF  __12F683
	#include "p12f683.inc"
	#define ADCpresent
  ENDIF

          
  IFDEF  __12F675
	#include "p12f675.inc"
	#define ADCpresent
  ENDIF

  IFDEF  __12F629
	#include "p12f629.inc"
	
  ENDIF
	errorlevel -302	; suppress banksel warning messages
  IFDEF __12F683	
     __CONFIG       _CP_OFF & _WDT_OFF & _BOD_OFF &_PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _CPD_OFF
  ELSE
     __CONFIG       _CP_ON & _WDT_OFF & _BODEN_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT & _MCLRE_OFF & _CPD_OFF
  ENDIF
       
; Program application GPR variables
; Note: GPR address space for 12F629/675 starts 0x20,
; ends at 0x5F and is not banked.
; --------------------------------------------
;          
	cblock 0x20 
	 start		;20	RxD byte 0 (packet start marker 0x81) 
	 address:0, command	;21	RxD byte 1 (address or command)
	 rdata:0,   comFunc	;22	RxD byte 2 (red data or command function)
	 gdata:0,   comData1	;23	RxD byte 3 (green data or command data1)
	 bdata:0,   comData2	;24	RxD byte 4 (blue data, or command data2)
 	 checksum		;25	RxD byte 5 (checksum)
	 deviceAddress		;26	device address
	 deviceMcast		;27	multiCast address
	 validate		;28	running hecksum of data received in each packet
	 bitTime		;29	counter used in the bit time delay function
	 pwmramp		;2A	running 'ramp' counter used by PWM function
	 writeMode		;2B	writeThrough or writeTo memory flag
	 rled		;2C	red   data, transferred to rpwm when pwmramp == 0
	 gled		;2D	green data, transferred to gpwm when pwmramp == 0
	 bled		;2E	blue  data, transferred to bpwm when pwmramp == 0
	 rpwm		;2F	red   LED PWM drive level [0=O%, 255=99%]
	 gpwm		;30	green LED PWM drive level [0=O%, 255=99%]
	 bpwm		;31	blue  LED PWM drive level [0=O%, 255=99%]
	 mem0:3		;32	RGB Memory 0 (3 bytes)
	 mem1:3		;35	RGB Memory 1 (3 bytes)
	 mem2:3		;38	RGB Memory 2 (3 bytes)
	 mem3:3		;3B	RGB Memory 3 (3 bytes)
	 mem4:3		;3E	RGB Memory 4 (3 bytes)
	 mem5:3		;41	RGB Memory 5 (3 bytes)
	 mem6:3		;44	RGB Memory 6 (3 bytes)
	 mem7:3		;47	RGB Memory 7 (3 bytes)
	 mem8:3		;4A	RGB Memory 8 (3 bytes)
	 mem9:3		;4D	RGB Memory 9 (3 bytes)
	 memA:3		;50	RGB Memory 10 (3 bytes)
	 memB:3		;53	RGB Memory 11 (3 bytes)
	 memC:3		;56	RGB Memory 12 (3 bytes)
	 memD:3		;59	RGB Memory 13 (3 bytes)
	 memE:3		;5C	RGB Memory 14 (3 bytes)
	 memoryPtr		;5F	RGB Memory address pointer
	endc


;        
; Define program constants
; -----------------------------------------
cSTARTDELAY	equ	.2	; delay after start bit detected before sampling 
			; next bit. Two is optimal
			; 1 =  0 - 22uS.
			; 2 = 22 - 44uS
			; 3 = 44 - 66uS

cMAXMEM	EQU	.15	; Number of RGB memories (limited by available GPR)
	           
; GPIO port definitions
; -----------------------------------------
; Edit the numerical values to change the I/O port functions
; The code will automatiaclly set the TRIS register correctly according to the values here.
; Internal weak pull-ups need to be configured manually in the 'Startup and initialisations" 
; section as needed.

rxd	EQU	.5	;in	Serial Received Data
unused4	EQU	.4	;in	
unused3	EQU	.3	;in  
blue	EQU	.2	;out 	Blue LED drive out
green	EQU	.1	;out	Green LED drive out
red	EQU	.0	;out 	Red LED drive out


; Bank Select pseudo instructions
#define	setbank0	bcf	STATUS,RP0	; Sel Bank 0
#define	setbank1	bsf	STATUS,RP0	; Sel Bank 1


; ******************************************************************************************************

	; Reset Vector
	; ------------
	org	0x000
_resetVector	goto	_startup

; ******************************************************************************************************
; Main code block
; Receive serial data and construct and validate 6 byte packets
; Drive RGB LED PWM outputs
;
; Asynchronous serial data is received on RxD input at 9600 bits per second
; Frames use 1 start, 8 data , 1 stop and no parity
; The software is written to receive a fixed length packet of 6 bytes
; The first frame is a start-of-packet delimiter, always 0x81
; The last byte is a checksum of the values in the previous five frames
; The checksum is computed by the sender to make validation as fast as possible for the receiver.
; When the received checksum is added to the sum of all five preceeding frames, the result should
; be zero. If not the packet is disgarded.
; 
; Packet format. 6 frames
; 0: start      = 0x81 start delimiter, always 0x81
; 1: address    = 0x01-0xFO: device or mulitcast address 	| 0xFE: command | 0xFF: broadcast
; 2: red        = 0x00-0xFF RED PWM value  		| command data byte 1
; 3: green      = 0x00-0xFF GREEN PWM value		| command data byte 2
; 4: blue       = 0x00-0xFF BLUE PWM value  		| command data byte 3
; 5: checksum	= 2's complement of sum [frames 0::5]
;
;       T01234567P
;     |_XXXXXXXX    Serial data frame starT D0-D7 stoP 
;     (Note: RS-232 inverts the levels: 0 = +3 to +15V, 1= -3 to -15V)
;
; ------------------------------------------------------------------------------------------------------

_newPacket	clrf 	validate	; clear packet checksum validation byte
	movlw	start	; get memory location of start variable (first byte of packet)
	movwf 	FSR	; write to FSR (indirection register)

_startBit	movlw	cSTARTDELAY	; use frame byte as temporary counter
	movwf	INDF	;
_waitStart	clrwdt		; clear watchdog timer
	call	_pwmDrive	; 
	btfsc	GPIO,rxd	; wait for start bit (0-20us from falling edge)
	goto	_waitStart	; 
	decfsz	INDF,F	; decrement temporary counter
	goto	_waitStart	; test start bit 2 times consecutively with 24uS interval
			; should now be between 48uS and 68uS from falling
			; edge of start bit depending on when it was detected
			; (bit length is 104uS)
			
			; frame byte is used as counter for the start bit code
			; so it always contains zero when we get here.
	bsf	INDF,7	; set bit 7 (used to detect 8th bit shifted in)

_nextBit	call	_waitBit	; wait full bit period to see middle(ish) of RxData bit
	rrf	INDF,F	; rotate right RxData byte and catch bit in Carry
	btfsc	GPIO,rxd	; test RxD line (serial bit)
	bsf	INDF,7	; if high set bit 7 of RxData byte
	skpc		; test carry to see if '1' rotated out
	goto	_nextBit	; if not wait for next bit
	
_stopBit	call	_waitBit	; wait 1 bit time
	btfss	GPIO,rxd	; test for 1 - is a stop bit
	goto	_newPacket	; if not 1, framing error, discard packet 
	
	movfw	start	; compare first byte received 
	xorlw	0x81	; with 0x81 (start-of-packet delimiter)
	bnz	_newPacket	; if not 0x81, restart packet receive

	movfw	INDF	; put last byte received in W
	addwf	validate,F	; add to validate variable and save 
	
	movfw	FSR	; compare FSR (indirection address pointer) with GPR memory...
	addlw	-(checksum)	; address of last byte (checksum) by subtraction
	bz	_validatePkt	; if they're the same (result 0), we received 6 bytes
	incf	FSR,F	; otherwise increment FSR to point to next byte
	goto	_startBit	; and get the next frame
	
_validatePkt	movf	validate,F	; validate has been summing all received frames, including checksum
	bz	_goodPacket	; if contents are now zero, the validation is good
	goto	_newPacket	; otherwise packet is bad, Disgard packet
	
			; --------------------------------------------------------------------------
			; Packet Address / Control byte 
			; 0x00 - 0x7F = single (unicast) device address
			; 0x80, 0x90, 0xA0, 0xB0, 0xC0, 0xD0, 0xE0, 0xF0 = group multicast address
			; 0xFE = command packet
			; 0xFF = broadcast address. All devices should accept this RGB data
			; 
_goodPacket	incf	address,W	; address=address+1, save in W (test for 0xFF - broadcast)
	bz	_thisDevice	; if result 0, address was 255 so goto _thisDevice
	
	addlw	.1	; add 1 to W (Test for 0xFE - command)
	bz	_command	; if address was 254, now 0 so goto _command

	movfw	address	; Load W with address
	xorwf	deviceAddress,W	; XOR with device address
	skpnz		; skip if not zero - address didn't match
	goto	_thisDevice	
	
	movfw	address	; load W with address
	xorwf	deviceMcast,W	; XOR with deviceMcast address
	skpz		; skip if result 0 - address did match
	goto	_newPacket	; if not get new Packet
	
			; --------------------------------------------------------------------------
_thisDevice     btfss	writeMode,0	; Test mode: 0 == writeThrough, !0 == WriteTo (active memory)
	goto	_writeThru

_writeTo	movfw	memoryPtr	; Load W with current memory pointer
	movwf	FSR	; save W into indirection pointer register (point to active memory Red address)
	
	movfw	rdata	; put received Red data into W
	movwf	INDF	; save W in active RGB memory
	incf	FSR,F	; point to active memory Green address
	movfw	gdata
	movwf	INDF
	incf	FSR,F	; point to active memory Red address
	movfw	bdata	; same as Red code but for Blue
	movwf	INDF
	goto	_newPacket

_writeThru	movfw	rdata	; put received Red data into W
	movwf	rled	; save W in Red LED latch 
	movfw	gdata	; same as Red code but for Green
	movwf	gled
	movfw	bdata	; same as Red code but for Blue
	movwf	bled
	goto	_newPacket



; ******************************************************************************************************
; Bit Time delay
; 9600bps = 104uS per bit
; this code block from call to return takes 98uS (+6uS if pwmRamp was zero)
; Code in the calling 'bit shift routine' takes this to 104uS / per bit exactly (110uS if pwmRamp was zero)
; ------------------------------------------------------------------------------------------------------
	
_waitBit	movlw	.4	;3
	movwf	bitTime	;4

_bitLoop	call	_pwmDrive	;6
	decfsz	bitTime,F	;21
	goto	_bitLoop	;23   repeat bit loop
	clrc	 	; must return with Carry bit clear

	return		;3   
; ******************************************************************************************************
; PWM code
; from call to return = 20uS
; compares current RG & B led values with 8 bit (256 step) pwmramp and activates GPIO output accordingly
; note: LED will never be 100% on. 
;       With xpwm=255 led will be off for 1 pwm period
;       or minimum mark/space ratio is 0%, maximum is 99%
;
; LED data is only transferred to the LED PWM registers when the PWM Ramp is at zero.
; This adds 6uS to a PWM Drive cycle once in every 256 calls to the pwmDrive function.
; When serial data is being received continually a transfer of LED data will occur every second packet
; received.
; 
; To reduce the effects of switching all three outputs on at the same time (pwm values are equal) 
; the RG & B outputs are enabled a minimum of 4uS apart.
; ------------------------------------------------------------------------------------------------------
;			;Time uSeconds
_pwmDrive	incf	pwmramp,F	;3 (call is 2 + incf = 3)
	skpz		;4 
	goto	_pwmEntry	;6

_transfer	clrf	GPIO	;
	movfw	rled	;  
	movwf	rpwm	;  
	movfw	gled	;  
	movwf	gpwm	;  
	movfw	bled	;  
	movwf	bpwm	;  

_pwmEntry	movfw	pwmramp	;7 
	addwf	bpwm,W	;8 
	skpnc		;9
	bsf	GPIO,blue	;10   
	movfw	pwmramp	;11
	addwf	gpwm,W	;12
	skpnc		;13
	bsf	GPIO,green	;14
	movfw	pwmramp	;15
	addwf	rpwm,W	;16
	skpnc		;17
	bsf	GPIO,red	;18
	return		;20 (return is 2 instruction cycles)
	

; ******************************************************************************************************
; EEPROM functions
; ------------------------------------------------------------------------------------------------------
; EEPROM read
; call with W reg containg address to read
; returns with eeprom read data in W reg
_eeRead	setbank1
	movwf	EEADR	
	bsf	EECON1,RD
	movf	EEDATA,W
	setbank0
	return

; EEPROM write
; calling code must set EEADR register and call with W reg containg data to write
; Note:
; According to the 12F627/675 datasheet, the maximum time for a EEPROM write/ease cycle 
; is 6mS.  Since the serial bit rate is 9600bps the bit period is 1/9600 seconds = 104uS,
; each frame contains 10 bits (start + 8 data + 1 stop bit) and each packet 6 frames.
; Therefore the time taken to receive each packet is 1/9600 x 6 x 10 = 6.25mS
; Since each command packet will only initiate a single write to the EEPROM we don't
; need to test for a completed EEPROM write cycle since we will never issue two write cycles
; less than 6.25mS apart.

_eeWrite	setbank1
	movwf	EEDATA
	bsf	EECON1,WREN
	movlw	0x55
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	setbank0
	return
	
; ******************************************************************************************************
; Command Function
; command (byte1)   = 0xFE -> command mode
;
; comFunc (byte2)   = 0x01  -> Memory transfer	    | Transfer RGB values from memory to LED transfer registers
;			    | comByte1 == memory [0::cMAXMEM]	
;
; comFunc (byte2)   = 0x02  -> Select memory	    | Select memory to store RGB values
;			    | comByte1 == memory [0::cMAXMEM]	
; 	
; comFunc (byte2)   = 0x04  -> Write Mode	    | Select Write Direct or Write Memory mode
;			    | comByte1 == 0 Write Direct
;			    | comByte1 != 0 Write Memory
;
; comFunc (byte2)   = 0x08  -> Soft reset	    | Unconditional jump to 0x000 (reset vector)
;
; comFunc (byte2)   = 0x10  -> Set device address   | if comByte2 == 0xFF then set deviceAddress = comByte1
;			    | if comByte2 == deviceAddress then set deviceAddress = comByte1
;			    | otherwise deviceAddress unchanged.
; ------------------------------------------------------------------------------------------------------
_command	btfsc	comFunc,0	; Test Bit 0 (1)
	goto	_cmdMemTrnf	; If 1 goto _cmdMemTrnf
	
	btfsc	comFunc,1	; Test Bit 1 (2)
	goto	_cmdSelMem	; If 1 goto _cmdSelMem
	
	btfsc	comFunc,2	; Test Bit 2 (4)
	goto	_cmdWrtMode	; If 1 goto _cmdWrtMode

	btfsc	comFunc,3	; Test Bit 3 (8)
	goto	_resetVector	; If 1 goto _resetVector
	
	btfsc	comFunc,4	; Test Bit 4 (16)
	goto	_cmdSetAddr	; If 1 goto _cmdSetAddr
	
	goto	_newPacket	; not a valid command, get new packet.
	
_cmdMemTrnf	movlw	-(cMAXMEM)	; load W with two's compliment of cMAXMEM constant
	addwf	comData1,W	; add to ComData1 (C = 1 if W > cMAXMEM)
	movfw	comData1	; load ComData1 into W
	bc	_newPacket	; if Carry set, W > cMAXMEM, get new packet
	rlf	comData1,W	; shift left (multiply by 2)
	addwf	comData1,W	; add original value (results in a fixed multiply by 3)
	addlw	mem0	; add address of memory base (to generate memory address offset)

	movwf	FSR	; Save W in FSR register
	movfw	INDF	; Load W with Red Memory value (by indirection)
	movwf	rled	; save to rled
	incf	FSR,F	; increment FSR, point to Green memory
	movfw	INDF	; Load W with Green memory value
	movwf	gled	; save to gled
	incf	FSR,F	; increment FSR, point to Blue memory
	movfw	INDF	; Load W with Blue memory value
	movwf	bled	; save to bled
	goto	_newPacket	; setup for next packet
	
_cmdSelMem	movlw	-(cMAXMEM)	; load W with two's compliment of cMAXMEM constant
	addwf	comData1,W	; add to ComData1 (C = 1 if W>cMAXMEM8)
	bc	_newPacket	; if Carry set, W > cMAXMEM, get new packet
	rlf	comData1,W	; shift left (multiply by 2)
	addwf	comData1,W	; add original value (gives a fixed multiply by 3)
	addlw	mem0	; add address of memory base
	movwf	memoryPtr	; save to memory pointer variable
	goto	_newPacket	; setup for next packet
	
	
_cmdWrtMode	clrf	writeMode	; clear writeMode flag variable
	movf	comData1,F	; test comData1 for non-zero value
	skpz		; skip next of comData1 is zero
	bsf	writeMode,0	; else set bit 0 of writeMode flag variable
	goto	_newPacket	;setup for next packet
	
	
_cmdSetAddr	btfsc	comData1,7	; Test bit 7 of new address to make sure it is clear
	goto	_newPacket	; if bit 7 is set, invalid device address so get next Packet
	movfw	comData2	; test if comByte2 == current device address
	xorwf	deviceAddress,W	; ...
	skpz		; if it is, apply new address
	incf	comData2,W	; if it isn't test if comByte2 == 0xFF
	skpz		; if it does, apply new address
	goto	_newPacket	; otherwise, get new packet
	
_setNewAddr	setbank1
	clrf	EEADR	; set EEPROM address to 0
	setbank0
	movfw	comData1	; comData contains new device address
	movwf	deviceAddress	; save it to deviceAddress variable
	movwf	deviceMcast	; save to deviceMcast variable
	call	_eeWrite	; save to EEPROM 
	movlw	0x70	; load W with bit mask
	andwf	deviceMcast,F	; mask out unwanted bits in Multicast address
	bsf	deviceMcast,7	; set bit 7 to a 1
	goto	_newPacket	; setup for next packet




; ******************************************************************************************************
; Initialisation and startup code block
; ------------------------------------------------------------------------------------------------------

_startup	clrf	GPIO	; clear GPIO output before setting TRIS register
	setbank1		; switch to register bank 1
	
  IFDEF OSCCAL			; defined only for 12F629 / 675
	call 	0x3FF	; read factory oscillator calibration value
	movwf	OSCCAL	; write to OSCCAL register
			; If using a 12F629/675 this value must be present
			; and correct, or the code will not function properly
			; if it functions at all.
  ENDIF

  IFDEF ADCpresent
	clrf	ANSEL	; Set ports for digital mode (12F675 / 12F683 only)
  ENDIF
	movlw	~(1<<red | 1<<green | 1<<blue ) 
	movwf	TRISIO	; Sets TRIS register
	
	; Set weak-pull up (enable as required here)
	; ---------------------------------------------------------------
	bcf	OPTION_REG,NOT_GPPU ; enable weak-pull up
	bsf	WPU,5	; enable weak-pull-up on GPIO5
			; GPIO3 has no weak-pull-up feature
  	setbank0
	movlw	0x07	; load W=7
  IFDEF __12F683
	movwf	CMCON0	; disable Comparator
  ELSE
	movwf	CMCON	; disable Comparator
  ENDIF

	; Clear GPR memory
	; ---------------------------------------------------------------
	
	movlw	0x20	; load W with address of first GPR memory
	movwf	FSR	; save in FSR indirection register
	movwf	0x5F	; save a non-zero value to last GPR memory
_clrNext	clrf	INDF	; clear contents of register pointed to by FSR
	incf	FSR,F	; increment FSR
	movf	0x5F,F	; Test value in last GPR memory location
	skpz		; skip next if last GPR has been cleared
	goto	_clrNext	; otherwise, repeat until done.
	
	; Initialise specific variables
	; ---------------------------------------------------------------
	movlw	mem0	; load address of mem0 into W
	movwf	memoryPtr	; save in memory pointer variable

	; Restore saved values from EEPROM
	; ---------------------------------------------------------------
	movlw	.0	; load W with EEPROM read address
	call	_eeRead	; read deviceAddress from EEPROM 
	movwf	deviceAddress	; save to deviceAddress variable
	andlw	0x70	; mask out unwanted bits
	iorlw	0x80	; set bit 7
	movwf	deviceMcast	; save Multicast address

	goto	_newPacket

;	------------------------------------------------------------------------
	; Embed code revision into program memory
	dt "RGBv2-102.asm 14/06/2006 "  
	dt "(c) Pete Griffiths 2006 "
	dt "info@picprojects.org.uk" 
;	------------------------------------------------------------------------
	fill 00, 0x3FF-$
; *************************************************************************************************************
; Configuration settings held in EEPROM
; These settings MUST be initialised when the device is programmed.
; The code does not validate them or initialise them if missing or incorrect.
;
	org 0x2100

	;EEPROM address 00. Device Address
	de	0x00	; deviceAddress
			; Valid setting 0x00 - 0x7F	

			
; *************************************************************************************************************			

	end

